==> release.12.2.28.1.tgz: cgi-cpn/prontus_update_real.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Actualizar Prontus a la version mas reciente
# ---------------------------------------------------------------

# -------------------------------BEGIN SCRIPT---------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Digest::MD5;
use Email::MIME;
use Email::Sender::Simple;
use Email::Sender::Transport::SMTP;
use Email::Simple;
use Encode;
use File::Compare;
use File::Copy;
use File::LibMagic;
use HTTP::Request;
use IO::All;
use LWP::UserAgent;
use LockFile::Simple;
use POSIX;
use Update;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_loading;
use lib_lock;
use lib_logproc;
use lib_mail;
use lib_prontus;
use lib_quota;
use lib_utils;
use lib_waitlock;
##/libs##

my ($PRONTUS_ID, $LOCK_FILE);
my $REMOTE_ADDR = '';
my $EMAIL_SOPORTE_PRONTUS = 'area_prontus@altavoz.net';

# ------------------------------------------------------------------------------
# MAIN.
# -------------
main:{
    lib_utils::setUtf8();
    $prontus_varglb::DIR_SERVER = $ARGV[0];
    $prontus_varglb::PRONTUS_ID = $ARGV[1];
    print STDERR "DIR_SERVER: [$prontus_varglb::DIR_SERVER]\n";
    print STDERR "PRONTUS_ID: [$prontus_varglb::PRONTUS_ID]\n";

    if (scalar @ARGV < 2) {
        print "Parametros insuficientes Terminando\n";
        exit;
    }

    if (! -d $prontus_varglb::DIR_SERVER) {
        lib_utils::write_log('Actualización de Prontus', 'Error: DIR_SERVER no válido.', '');
        print STDERR "ERROR: DIR_SERVER no válido[$prontus_varglb::DIR_SERVER]\n";
        exit;
    }

    my $msg_load = lib_utils::load_config_by_id($prontus_varglb::PRONTUS_ID);
    if ($msg_load ne '') {
        lib_utils::write_log('Actualización de Prontus', "Error: $msg_load.", '');
        print STDERR "Error: $msg_load\n";
        exit;
    }

    if ($ARGV[2]) {
        $REMOTE_ADDR = $ARGV[2];
        $REMOTE_ADDR =~ s/[^.0-9]+//g;
    }

    # Para el manejo del log de procesamiento
    $lib_logproc::LOG_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/prontus_update_log.html";

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTUALIZACIONES'} ne 'SI') {
        lib_utils::write_log('Actualización de Prontus', 'Error: actualizaciones automáticas deshabilitadas.', '');
        lib_logproc::add_to_log_finish("Error: las actualizaciones automáticas se encuentran deshabilitadas por configuración del producto.", 1);
    };

    # Bloqueo
    $LOCK_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/semaforo_prontus_update";

    # Detecta semaforo.
    my $lock_obj = &lib_lock::lock_file($LOCK_FILE);
    if (!ref $lock_obj) { # si ya tiene un bloqueo anterior, aborta.
        lib_utils::write_log('Actualización de Prontus', 'Error: Actualización anterior ejecutándose.', '');
        &lib_loading::finish_loading(0, "Proceso en ejecución. Por favor espere hasta que la actualización anterior termine.");
        lib_logproc::add_to_log_finish("Proceso en ejecución. Por favor espere hasta que la actualización anterior termine.", 1);
    };

    my $ret = &lib_loading::init('result_prontus_update.json');
    unless($ret) {
        &lib_loading::finish_loading(0, 'No se pudo escribir el archivo de respuesta');
        lib_logproc::add_to_log_finish('No se pudo escribir el archivo de respuesta', 1);
    }

    # Se realiza el proceso y se escribe al Log
    lib_logproc::flush_log();
    lib_logproc::writeRule();
    lib_logproc::add_to_log_count("INICIANDO PROCESO DE ACTUALIZACION");
    lib_utils::write_log('Actualización de Prontus', 'Inicio de actualización', '');
    my $error_msg = &actualizar_prontus();
    my ($respuesta, $mensaje);
    if ($error_msg eq '') {
        $respuesta = 1;
        $mensaje = "Actualización finalizada OK";
        lib_utils::write_log('Actualización de Prontus', $mensaje, '');
    } else {
        $respuesta = 0;

        $mensaje = "<strong>Actualización finalizada con errores:</strong><br/>$error_msg";
        lib_utils::write_log('Actualización de Prontus', "Error: $mensaje", '');
        &enviar_mail_error($EMAIL_SOPORTE_PRONTUS, $mensaje);
    };

    # Borra cache de no publicados
    glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/data/cache");

    # borramos archivos de status de tareas post update para que se ejecuten en el proximo login
    unlink "$prontus_varglb::DIR_SERVER/prontus/cpan/data/prontus_updated.txt";
    unlink "$prontus_varglb::DIR_SERVER/prontus/cpan/data/prontus_updated.json";

    # Elimina el bloqueo y termina ejecucion.
    &lib_lock::unlock_file($lock_obj, $LOCK_FILE);

    lib_logproc::add_to_log_count($mensaje);
    lib_logproc::writeRule();

    &lib_loading::finish_loading($respuesta, $mensaje);
    print STDERR "$mensaje\n";

}; # main

# ------------------------------------------------------------------------------
# SUB-RUTINAS.
# ------------------------------------------------------------------------------

# Envía un email con los errores encontrados al actualizar
# @param string $email el destinatario
# @param string $texto el mensaje.
# @return el resultado de lib_mail::enviar_mail().
sub enviar_mail_error {
    my ($email, $texto) = @_;
    my $from = 'prontus@' . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'};
    my $replyto_name = 'Prontus CMS';
    my $replyto_email = $EMAIL_SOPORTE_PRONTUS;
    my $server = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    if ($server =~ /^[0-9]+\.[0-9]+\./) {
        my $dir_server_ofusc = $prontus_varglb::DIR_SERVER;
        $dir_server_ofusc =~ s/\//-/g;
        $dir_server_ofusc =~ s/^[- ]+//g;
        $server .= " ($dir_server_ofusc)";
    }
    my $asunto = "[prontus-update] Error actualizando en $server - prontus_id: $prontus_varglb::PRONTUS_ID";

    my $buffer_completo_log = glib_fildir_03::read_file($lib_logproc::LOG_FILE);

    $texto .= "\n\nUser Log:\n\n$buffer_completo_log";

    my $buffer_error_log = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_error_log/prontus_update_real.pl.error.log");

    $texto .= "\n\nProntus error Log:\n\n$buffer_error_log";
    my $htmldoc = $texto;
    my $attach = '';
    my $url = '';
    my $dir_attach = '';
    my $smtp = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'};

    lib_mail::enviar_mail($email, $from, $replyto_name, $replyto_email, $asunto, $texto, $htmldoc, $attach, $url, $dir_attach, $smtp);
};

# ------------------------------------------------------------------------------
# Notifica al area prontus la ocurrencia del update
# @param string $email
# @param string $old_version
# @param string $new_version
# @param string $str_ok
# @return el resultado de lib_mail::enviar_mail().
sub enviar_mail_notif {
    my ($email, $old_version, $new_version, $str_ok) = @_;
    my $from = 'prontus@' . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'};
    my $replyto_name = 'Prontus CMS';
    my $replyto_email = $EMAIL_SOPORTE_PRONTUS;
    my $server = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    if ($server =~ /^[0-9]+\.[0-9]+\./) {
        my $dir_server_ofusc = $prontus_varglb::DIR_SERVER;
        $dir_server_ofusc =~ s/\//-/g;
        $dir_server_ofusc =~ s/^[- ]+//g;
        $server .= " ($dir_server_ofusc)";
    }

    my $asunto = "[prontus-update] Update OK en $server - prontus_id: $prontus_varglb::PRONTUS_ID";
    my $texto = '';
    my $htmldoc = 'El update fue ejecutado desde la IP: ' . $REMOTE_ADDR
                  . "<br>Release anterior: $old_version<br>Release actual: $new_version<br><br>"
                  . $str_ok;
    my $attach = '';
    my $url = '';
    my $dir_attach = '';
    my $smtp = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'};

    lib_mail::enviar_mail($email, $from, $replyto_name, $replyto_email, $asunto, $texto, $htmldoc, $attach, $url, $dir_attach, $smtp);
}

# ------------------------------------------------------------------------------
# Intenta la actualización y escribe a logs el proceso y el resultado.
# @param void
# @return string '' o un mensaje de error.
sub actualizar_prontus {
    my ($msg, $quota_usado, $quota_asignado, $usado_porc, $nousado_porc) = lib_quota::calcula_unix();

    &lib_loading::update_loading(100, 5);
    my $msg_abort = 'No fue posible actualizar Prontus';

    # Creacion del objeto Update (todas los params son obligatorios).
    my $upd_obj = Update->new(
                    'prontus_id'        => $prontus_varglb::PRONTUS_ID,
                    'version_prontus'   => $prontus_varglb::VERSION_PRONTUS,
                    'document_root'     => $prontus_varglb::DIR_SERVER)
                    || lib_logproc::add_to_log_finish("$msg_abort\n\nError inicializando Update: [$Update::ERR]", 1);

    $upd_obj->set_info_quota($quota_asignado, $quota_usado, $usado_porc);

    # Se chequea si es factible realizar la actualización
    &lib_loading::update_loading(100, 10);
    $upd_obj->descarga_upd_descriptor();
    if (!$upd_obj->update_disponible()) {
        return "Error: No se han detectado actualizaciones disponibles.";
    }

    # Detecta instancias de prontus compatibles con las cgis
    &lib_loading::update_loading(100, 20);
    lib_logproc::add_to_log_count("Detectando instancias Prontus ...");

    lib_logproc::add_to_log_count("Calculando espacio disponible ...");
    my $ret = $upd_obj->check_before_download();
    return "Error al calcular espacio libre en disco: [$Update::ERR]" if (!$ret);

    # Descarga release
    &lib_loading::update_loading(100, 30);
    lib_logproc::add_to_log_count("Descargando release: " . $upd_obj->{last_version_disponible} . ' desde ' . $upd_obj->{update_server} . '/release ...');

    $ret = $upd_obj->descarga_release();

    return "Error al descargar release: [$Update::ERR]" if (!$ret);
    lib_logproc::add_to_log_count("Release descargada ok en /_prontus_update/downloads/" . $upd_obj->{last_version_disponible});

    # Se chequea factibilidad de instalacion
    &lib_loading::update_loading(100, 70);
    $ret = $upd_obj->check_before_update();
    return "Error al hacer la validación: [$Update::ERR]" if (!$ret);

    # Se realizan los respaldos de todos los elementos
    &lib_loading::update_loading(100, 75);
    $ret = $upd_obj->crea_respaldos();
    return "Error al crear los respaldos: [$Update::ERR]" if (!$ret);

    # Instala CGIs de prontus
    &lib_loading::update_loading(100, 80);
    lib_logproc::add_to_log_count("Instalando nuevas CGIs");
    $ret = $upd_obj->install_cgis();
    return "Error al instalar CGIs: [$Update::ERR]" if (!$ret);
    lib_logproc::add_to_log_count("CGIs instaladas OK");

    # Actualiza core de prontus cpan y front
    &lib_loading::update_loading(100, 85);
    lib_logproc::add_to_log_count("Actualizando carpeta 'core' de Prontus");
    $ret = $upd_obj->install_core();
    return "Error al instalar Core: [$Update::ERR]" if (!$ret);
    lib_logproc::add_to_log_count("Carpeta 'core' de Prontus actualizada OK");

    # Actualiza /wizard_prontus/core
    &lib_loading::update_loading(100, 90);
    lib_logproc::add_to_log_count("Actualizando carpeta '/wizard_prontus/core'");
    $ret = $upd_obj->install_core_wizard();
    return "Error al instalar Core del Wizard: [$Update::ERR]" if (!$ret);
    lib_logproc::add_to_log_count("Carpeta '/wizard_prontus/core' actualizada OK");

    # Borra files descargados
    &lib_loading::update_loading(100, 95);
    $upd_obj->garbage_dirs();

    my $du_result;
    my $dir4du = "$prontus_varglb::DIR_SERVER/_prontus_update";
    $du_result = `du -d 2 -h $dir4du`; # estilo unix/freebsd
    if ($du_result eq '') {
        $du_result = `du -h $dir4du --max-depth=2`; # estilo linux
    }
    $du_result =~ s/$prontus_varglb::DIR_SERVER//g;

    my $str_ok = "La actualización ha finalizado OK.<br>"
               . "- Las carpetas y archivos actualizados fueron respaldados en /_prontus_update/updated/.<br>"
               . "- En caso de contingencia, también se preserva el update descargado, en /_prontus_update/downloads/.<br>"
               . "- En ambos casos, se mantienen automáticamente los últimos 3 respaldos de cada tipo,<br>eliminando los más antiguos e impidiendo así su acumulación.<br>"
               . "<br>El espacio utilizado actualmente por estos respaldos es el siguiente:<br><pre>$du_result</pre>";
    lib_logproc::add_to_log_count($str_ok);

    &enviar_mail_notif($EMAIL_SOPORTE_PRONTUS, $prontus_varglb::VERSION_PRONTUS, $upd_obj->{last_version_disponible}, $str_ok);
    &lib_loading::update_loading(100, 100);

    return '';
}

==> release.12.3.1.2.tgz: cgi-cpn/prontus_update_real.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Actualizar Prontus a la version mas reciente
# ---------------------------------------------------------------

# -------------------------------BEGIN SCRIPT---------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Digest::MD5;
use Email::MIME;
use Email::Sender::Simple;
use Email::Sender::Transport::SMTP;
use Email::Simple;
use Encode;
use File::Compare;
use File::Copy;
use File::LibMagic;
use HTTP::Request;
use IO::All;
use JSON;
use LWP::UserAgent;
use LockFile::Simple;
use POSIX;
use Update;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_lang;
use lib_loading;
use lib_lock;
use lib_logproc;
use lib_mail;
use lib_ports;
use lib_prontus;
use lib_quota;
use lib_utils;
use lib_waitlock;
##/libs##

my ($PRONTUS_ID, $LOCK_FILE);
my $REMOTE_ADDR = '';
my $EMAIL_SOPORTE_PRONTUS = 'area_prontus@altavoz.net';

# ------------------------------------------------------------------------------
# MAIN.
# -------------
main:{
    lib_utils::setUp();

    $prontus_varglb::DIR_SERVER = $ARGV[0];
    my $prontus_id = $ARGV[1];
    print STDERR "DIR_SERVER: [$prontus_varglb::DIR_SERVER]\n";
    print STDERR "PRONTUS_ID: [$prontus_id]\n";

    if (scalar @ARGV < 2) {
        print lib_lang::get_message('global_insufficient_arguments');
        exit;
    }
    my $update_label = lib_lang::get_message('global_prontus_update');
    my $err_dir_server = lib_lang::get_message('global_error_invalid_dir');
    if (! -d $prontus_varglb::DIR_SERVER) {
        lib_utils::write_log($update_label, $err_dir_server, '');
        print STDERR $err_dir_server."[$prontus_varglb::DIR_SERVER]\n";
        exit;
    }

    my $msg_load = lib_utils::load_config_by_id($prontus_id);
    if ($msg_load ne '') {
        lib_utils::write_log($update_label, lib_lang::get_interpolated_message('global_simple_error_int', $msg_load), '');
        print STDERR "$msg_load\n";
        exit;
    }

    if ($ARGV[2]) {
        $REMOTE_ADDR = $ARGV[2];
        $REMOTE_ADDR =~ s/[^.0-9]+//g;
    }

    # Para el manejo del log de procesamiento
    $lib_logproc::LOG_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/prontus_update_log.html";

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTUALIZACIONES'} ne 'SI') {
        lib_utils::write_log($update_label, lib_lang::get_message('update_real_updates_disabled'), '');
        lib_logproc::add_to_log_finish(lib_lang::get_message("update_real_updates_disabled_log"), 1);
    };

    # Bloqueo
    $LOCK_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/semaforo_prontus_update";

    # Detecta semaforo.
    my $lock_obj = &lib_lock::lock_file($LOCK_FILE);
    if (!ref $lock_obj) { # si ya tiene un bloqueo anterior, aborta.
        lib_utils::write_log($update_label, lib_lang::get_message('update_real_update_running'), '');
        my $msg_running=lib_lang::get_message('update_real_process_running');
        &lib_loading::finish_loading(0, $msg_running);
        lib_logproc::add_to_log_finish($msg_running, 1);
    };

    my $ret = &lib_loading::init('result_prontus_update.json');
    unless($ret) {
        my $msg_err=lib_lang::get_message('update_real_error_writing_response');
        &lib_loading::finish_loading(0, $msg_err);
        lib_logproc::add_to_log_finish($msg_err, 1);
    }

    # Se realiza el proceso y se escribe al Log
    lib_logproc::writeRule();
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_starting"));
    lib_utils::write_log($update_label, lib_lang::get_message('update_real_update_start'), '');
    my $error_msg = &actualizar_prontus();
    my ($respuesta, $mensaje);
    if ($error_msg eq '') {
        $respuesta = 1;
        $mensaje = lib_lang::get_message('update_real_end_ok');
        lib_utils::write_log($update_label, $mensaje, '');
    } else {
        $respuesta = 0;

        $mensaje = lib_lang::get_interpolated_message('update_real_end_error', $error_msg);
        lib_utils::write_log($update_label, lib_lang::get_interpolated_message('global_simple_error_int', $mensaje), '');
        &enviar_mail_error($EMAIL_SOPORTE_PRONTUS, $mensaje);
    }

    # Borra cache de no publicados
    lib_ports::borrar_json_artic_pubs($prontus_varglb::PRONTUS_ID);

    # borramos archivos de status de tareas post update para que se ejecuten en el proximo login
    unlink "$prontus_varglb::DIR_SERVER/prontus/cpan/data/prontus_updated.txt";
    unlink "$prontus_varglb::DIR_SERVER/prontus/cpan/data/prontus_updated.json";

    # Elimina el bloqueo y termina ejecucion.
    &lib_lock::unlock_file($lock_obj, $LOCK_FILE);

    lib_logproc::add_to_log_count($mensaje);
    lib_logproc::writeRule();

    &lib_loading::finish_loading($respuesta, $mensaje);
    print STDERR "$mensaje\n";

}; # main

# ------------------------------------------------------------------------------
# SUB-RUTINAS.
# ------------------------------------------------------------------------------

# Envía un email con los errores encontrados al actualizar
# @param string $email el destinatario
# @param string $texto el mensaje.
# @return el resultado de lib_mail::enviar_mail().
sub enviar_mail_error {
    my ($email, $texto) = @_;
    my $from = 'prontus@' . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'};
    my $replyto_name = 'Prontus CMS';
    my $replyto_email = $EMAIL_SOPORTE_PRONTUS;
    my $server = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    if ($server =~ /^[0-9]+\.[0-9]+\./) {
        my $dir_server_ofusc = $prontus_varglb::DIR_SERVER;
        $dir_server_ofusc =~ s/\//-/g;
        $dir_server_ofusc =~ s/^[- ]+//g;
        $server .= " ($dir_server_ofusc)";
    }
    my $asunto = "[prontus-update] Error actualizando en $server - prontus_id: $prontus_varglb::PRONTUS_ID";

    my $buffer_completo_log = glib_fildir_03::read_file($lib_logproc::LOG_FILE);

    $texto .= "\n\nUser Log:\n\n$buffer_completo_log";

    my $buffer_error_log = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_error_log/prontus_update_real.pl.error.log");

    $texto .= "\n\nProntus error Log:\n\n$buffer_error_log";
    my $htmldoc = $texto;
    my $attach = '';
    my $url = '';
    my $dir_attach = '';
    my $smtp = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'};

    lib_mail::enviar_mail($email, $from, $replyto_name, $replyto_email, $asunto, $texto, $htmldoc, $attach, $url, $dir_attach, $smtp);
};

# ------------------------------------------------------------------------------
# Notifica al area prontus la ocurrencia del update
# @param string $email
# @param string $old_version
# @param string $new_version
# @param string $str_ok
# @return el resultado de lib_mail::enviar_mail().
sub enviar_mail_notif {
    my ($email, $old_version, $new_version, $str_ok) = @_;
    my $from = 'prontus@' . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'};
    my $replyto_name = 'Prontus CMS';
    my $replyto_email = $EMAIL_SOPORTE_PRONTUS;
    my $server = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    if ($server =~ /^[0-9]+\.[0-9]+\./) {
        my $dir_server_ofusc = $prontus_varglb::DIR_SERVER;
        $dir_server_ofusc =~ s/\//-/g;
        $dir_server_ofusc =~ s/^[- ]+//g;
        $server .= " ($dir_server_ofusc)";
    }

    my $asunto = "[prontus-update] Update OK en $server - prontus_id: $prontus_varglb::PRONTUS_ID";
    my $texto = '';
    my $htmldoc = 'El update fue ejecutado desde la IP: ' . $REMOTE_ADDR
                  . "<br>Release anterior: $old_version<br>Release actual: $new_version<br><br>"
                  . $str_ok;
    my $attach = '';
    my $url = '';
    my $dir_attach = '';
    my $smtp = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'};

    lib_mail::enviar_mail($email, $from, $replyto_name, $replyto_email, $asunto, $texto, $htmldoc, $attach, $url, $dir_attach, $smtp);
}

# ------------------------------------------------------------------------------
# Intenta la actualización y escribe a logs el proceso y el resultado.
# @param void
# @return string '' o un mensaje de error.
sub actualizar_prontus {
    my ($msg, $quota_usado, $quota_asignado, $usado_porc, $nousado_porc) = lib_quota::calcula_unix();

    &lib_loading::update_loading(100, 5);
    my $msg_abort = lib_lang::get_message('update_real_abort');

    # Creacion del objeto Update (todas los params son obligatorios).
    my $upd_obj = Update->new(
                    'prontus_id'        => $prontus_varglb::PRONTUS_ID,
                    'version_prontus'   => $prontus_varglb::VERSION_PRONTUS,
                    'document_root'     => $prontus_varglb::DIR_SERVER)
                    || lib_logproc::add_to_log_finish("$msg_abort\n\n".lib_lang::get_interpolated_message('global_error_update_obj', $Update::ERR), 1);

    $upd_obj->set_info_quota($quota_asignado, $quota_usado, $usado_porc);

    # Se chequea si es factible realizar la actualización
    &lib_loading::update_loading(100, 10);
    $upd_obj->descarga_upd_descriptor();
    if (!$upd_obj->update_disponible()) {
        return lib_lang::get_message("update_real_no_update_available");
    }

    # Detecta instancias de prontus compatibles con las cgis
    &lib_loading::update_loading(100, 20);
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_detecting_instances"));

    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_calculating_space"));
    my $ret = $upd_obj->check_before_download();
    return lib_lang::get_interpolated_message("update_real_error_calculating_space", $Update::ERR) if (!$ret);

    # Descarga release
    &lib_loading::update_loading(100, 30);
    lib_logproc::add_to_log_count(lib_lang::get_interpolated_message('update_real_downloading_from', $upd_obj->{last_version_disponible}, $upd_obj->{update_server}));

    $ret = $upd_obj->descarga_release();

    return lib_lang::get_interpolated_message("update_real_error_downloading", $Update::ERR) if (!$ret);
    lib_logproc::add_to_log_count(lib_lang::get_interpolated_message('update_real_release_downloaded', $upd_obj->{last_version_disponible}));

    # Se chequea factibilidad de instalacion
    &lib_loading::update_loading(100, 70);
    $ret = $upd_obj->check_before_update();
    return lib_lang::get_interpolated_message("update_real_error_validating", $Update::ERR) if (!$ret);

    # Se realizan los respaldos de todos los elementos
    &lib_loading::update_loading(100, 75);
    $ret = $upd_obj->crea_respaldos();
    return lib_lang::get_interpolated_message('update_real_error_backup', $Update::ERR) if (!$ret);

    # Instala CGIs de prontus
    &lib_loading::update_loading(100, 80);
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_installing_cgis"));
    $ret = $upd_obj->install_cgis();
    return lib_lang::get_interpolated_message('update_real_error_cgis', $Update::ERR) if (!$ret);
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_cgis_installed"));

    # Actualiza core de prontus cpan y front
    &lib_loading::update_loading(100, 85);
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_updating_dir", 'core'));
    $ret = $upd_obj->install_core();
    return lib_lang::get_interpolated_message('update_real_error_updating_dir', 'Core', $Update::ERR) if (!$ret);
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_updated_dir", 'core'));

    # Actualiza /wizard_prontus/core
    &lib_loading::update_loading(100, 90);
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_updating_dir", '/wizard_prontus/core'));
    $ret = $upd_obj->install_core_wizard();
    return lib_lang::get_interpolated_message('update_real_error_updating_dir', 'Core del Wizard', $Update::ERR) if (!$ret);
    lib_logproc::add_to_log_count(lib_lang::get_message("update_real_updated_dir", 'wizard_prontus/core'));

    # Borra files descargados
    &lib_loading::update_loading(100, 95);
    $upd_obj->garbage_dirs();

    my $du_result;
    my $dir4du = "$prontus_varglb::DIR_SERVER/_prontus_update";
    $du_result = `du -d 2 -h $dir4du`; # estilo unix/freebsd
    if ($du_result eq '') {
        $du_result = `du -h $dir4du --max-depth=2`; # estilo linux
    }
    $du_result =~ s/$prontus_varglb::DIR_SERVER//g;

    my $str_ok = lib_lang::get_interpolated_message('update_real_str_ok', $du_result);
    lib_logproc::add_to_log_count($str_ok);

    &enviar_mail_notif($EMAIL_SOPORTE_PRONTUS, $prontus_varglb::VERSION_PRONTUS, $upd_obj->{last_version_disponible}, $str_ok);
    &lib_loading::update_loading(100, 100);

    return '';
}
